package game.playState {
	import game.Assets;
	import game.Constants;
	import game.startState.StartState;
	
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.World;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.tweens.misc.VarTween;
	import net.flashpunk.utils.Ease;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.utils.MouseButton;
	
	public class PlayState extends World {
		
		protected var _running:Boolean;
		protected var _gameIsOver:Boolean;
		protected var _paused:Boolean;
		
		public var player:Player;
		
		private var _points:Number = 0;
		public var textPoints:Text;
		
		public var startScreenEntity:Entity;

		public var roundStarted:Boolean;
		
		protected var pausedOverlay:Overlay;
		protected var pausedTextEntity:Entity;
		protected var pausedInstructionsText:Text;
		protected var pausedInstructionsTextEntity:Entity;
		
		protected var controlScheme:uint;
		
		public function get running():Boolean { return _running; }
		public function get gameIsOver():Boolean { return _gameIsOver; }
		public function get paused():Boolean { return _paused; }
		
		public function PlayState(cameFromStartScreen:Boolean, controlScheme:uint) {
			super();
			
			this.controlScheme = controlScheme;
			
			//var backgroundEntity:Entity = addGraphic(new Image(Assets.BACKGROUND));
			//backgroundEntity.layer = Constants.LAYER_BACKGROUND;
			
			//add(new Cursor(this));
			
			player = new Player(this, FP.width / 2, FP.height / 2, Key.A, Key.D, Key.W, Key.S, 0, 0, 0, 0, 0, 0, 0, 0, Key.SPACE, MouseButton.LEFT, MouseButton.RIGHT, MouseButton.MIDDLE);
			
			add(player);
			
			textPoints = new Text("Points: 0");
			textPoints.x = 0;
			textPoints.y = 0;
			textPoints.color = 0xFFFFFF;
			textPoints.size = 16;
			textPoints.outlineColor = 0;
			textPoints.outlineStrength = 4;
			addGraphic(textPoints, Constants.LAYER_TEXT_OVER_OVERLAY);
			
			_points = 0;
			addPoints(0);
			
			if (cameFromStartScreen)
			{
				var startScreenImage:Image = new Image(Assets.START_BACKGROUND);
				startScreenEntity = addGraphic(startScreenImage, Constants.LAYER_ULTMATE_OVERLAY);
				
				var backgroundOut:VarTween = new VarTween(startScreenFadeOutComplete);
				backgroundOut.tween(startScreenImage, "alpha", 0, 1, Ease.quadIn);
				addTween(backgroundOut);
			}

			add(new Hud(this));
			
			pausedOverlay = new Overlay(0.5);
			pausedOverlay.visible = false;
			add(pausedOverlay);
			
			var pausedText:Text = new Text("Paused");
			pausedText.size = 32;
			pausedText.x = FP.width / 2 - pausedText.width / 2;
			pausedText.y = FP.height / 2 - pausedText.height / 2;
			pausedText.color = 0xFFFFFF;
			pausedText.outlineColor = 0;
			pausedText.outlineStrength = 4;
			pausedTextEntity = addGraphic(pausedText, Constants.LAYER_TEXT_OVER_OVERLAY);
			pausedTextEntity.visible = false;

			pausedInstructionsText = new Text("");
			pausedInstructionsText.size = 16;
			pausedInstructionsText.y = pausedText.y + pausedText.height + 10;
			pausedInstructionsText.align = "center";
			pausedInstructionsText.color = 0xFFFFFF;
			pausedInstructionsText.outlineColor = 0;
			pausedInstructionsText.outlineStrength = 4;
			pausedInstructionsTextEntity = addGraphic(pausedInstructionsText, Constants.LAYER_TEXT_OVER_OVERLAY);
			pausedInstructionsTextEntity.visible = false;

			_running = true;
			_gameIsOver = false;
			_paused = false;
			
			var i:int;
			var name:String;
			
			roundStarted = false;
		}
		
		/*
		private function startMusic():void
		{
			if (!Constants.DEBUG)
			{
				bgMusic = new Sfx(Assets.MUSIC_BACKGROUND_1, onMusicComplete);
				bgMusic.play();
			}
		}
		
		private function onMusicComplete():void
		{
			musicState++;
			if (musicState == 2)
			{
				bgMusic.stop();
				bgMusic = new Sfx(Assets.MUSIC_BACKGROUND_2, onMusicComplete);
				bgMusic.play();
			}
			else if (musicState == 3)
			{
				bgMusic.stop();
				bgMusic = new Sfx(Assets.MUSIC_BACKGROUND_3);
				bgMusic.loop();
			}
		}
		*/
		
		private function startScreenFadeOutComplete():void
		{
			remove(startScreenEntity);
		}
		
		override public function update():void {
			if (gameIsOver)
			{
				if (Input.pressed(Key.ENTER) || Input.pressed(Key.R))
				{
					//bgAmbient.stop();
					FP.world = new PlayState(false, controlScheme);
					return;
				}
			}
			else
			{
				if (paused)
				{
					if (Input.pressed(Key.ESCAPE))
					{
						submitScore();
						FP.world = new StartState();
						return;
					}
					
					if (Input.pressed(Key.ANY) || Input.mousePressed || Input.mousePressedRight || Input.mousePressedMiddle)
					{
						resumeGame();
					}
					
					return;
				}
				else if (Input.pressed(Key.P) || Input.pressed(Key.ESCAPE))
				{
					pauseGame(false);
					return;
				}
			}
			
			super.update();
			
			if (!running)
				return;
			
			if (!roundStarted)
				return;
			
			if (Constants.SCORE_PER_SECOND > 0)
			{
				addPoints(FP.elapsed * Constants.SCORE_PER_SECOND);
			}
		}
		
		/*
		override public function focusLost():void
		{
			super.focusLost();
			pauseGame(true);
		}
		
		override public function focusGained():void
		{
			super.focusGained();
			resumeGame();
		}
		*/
		
		public function pauseGame(focusWasLost:Boolean):void
		{
			if (paused || gameIsOver)
				return;
			
			_paused = true;
			
			if (focusWasLost)
			{
				pausedInstructionsText.text = "Click anywhere to resume playing.";
			}
			else
			{
				pausedInstructionsText.text = "Press Escape to go back to the menu.\nClick or press any other key to resume playing.";				
			}
			pausedInstructionsText.x = FP.width / 2 - pausedInstructionsText.width / 2;			
			
			pausedOverlay.visible = true;
			pausedTextEntity.visible = true;
			pausedInstructionsTextEntity.visible = true;
			
			/*
			if (bgMusic)
				bgMusic.stop();
			
			if (bgAmbient)
				bgAmbient.stop();
			*/
		}
		
		public function resumeGame():void
		{
			if (!paused || gameIsOver)
				return;
			
			_paused = false;
			
			pausedOverlay.visible = false;
			pausedTextEntity.visible = false;
			pausedInstructionsTextEntity.visible = false;
			
			/*
			if (bgMusic)
				bgMusic.resume();
			
			if (bgAmbient)
				bgAmbient.resume();
			*/
		}
		
		public function gameOver(message:String):void
		{
			resumeGame();
			_running = false;
			_gameIsOver = true;
			
			var gameOverText:Text = new Text(message + "\nPress Enter or R to restart.");
			gameOverText.color = 0xFFFFFF;
			gameOverText.size = 32;
			gameOverText.align = "center";
			gameOverText.outlineColor = 0;
			gameOverText.outlineStrength = 4;
			gameOverText.x = FP.width / 2 - gameOverText.width / 2;
			gameOverText.y = FP.height / 2 - gameOverText.height / 2;
			addGraphic(gameOverText, Constants.LAYER_TEXT_OVER_OVERLAY);
			
			add(new Overlay(0.5));
			
			/*
			if (bgMusic != null)
			{
				bgMusic.stop();
			}
			*/
			
			submitScore();
		}

		protected function submitScore():void
		{
			if (Constants.kongregate == null)
				return;

			//Constants.kongregate.stats.submit(Constants.STAT_HIGHSCORE, Math.floor(_points));
		}
		
		public function addPoints(amount:Number):void
		{
			_points += amount;
			
			textPoints.text = "Points: " + Math.floor(_points);
		}
		
		override public function add(e:Entity):Entity {
			/*
			if (e is Egg) {
				eggs.push(e);
			}
			*/

			return super.add(e);
		}
		
		override public function remove(e:Entity):Entity {
			var index:int;
			
			/*
			if (e is Egg) {
				index = eggs.indexOf(e);
				if (index != -1)
					eggs.splice(index, 1);
			}
			*/
			
			return super.remove(e);
		}
	}
}