package game.playState
{
	import flash.geom.Point;
	
	import game.Constants;
	
	import net.flashpunk.FP;
	import net.flashpunk.Tween;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.tweens.misc.VarTween;
	import net.flashpunk.utils.Draw;
	import net.flashpunk.utils.Ease;
	import net.flashpunk.utils.Input;
	
	public class Player extends PlaystateEntity
	{
		protected static const INSTRUCTION_TEXT_START_Y:Number = 10;
		protected static const INSTRUCTION_TEXT_DISTANCE_Y:Number = 5;
		
		protected static const CONTACT_POINT_HANDS_DISTANCE:Number = 35;
		protected static const CONTACT_POINTS_SLASH_DISTANCES_FORWARD:Array = [10, 30, 50];
		protected static const CONTACT_POINTS_SLASH_DISTANCES_RIGHT:Array = [-30, 0, 30];
		protected static const FAIL_TEXT_COLOR:uint = 0xFF0000;
		
		protected var speed:Number = Constants.PLAYER_SPEED;

		protected var keyLeft:int;
		protected var keyRight:int;
		protected var keyUp:int;
		protected var keyDown:int;
		protected var keyActionSlash:int;
		protected var keyActionShoot:int;
		protected var keyActionPickup:int;
		/*
		protected var mouseActionMove:int;
		protected var mouseActionTurn:int;
		protected var mouseActionSlash:int;
		protected var mouseActionShoot:int;
		protected var mouseActionPickup:int;
		*/
		
		protected var nextInstructionTextStartY:Number;
		protected var instructionsTextFadingOut:Boolean;
		protected var instructionsMovement:CompoundTextEntity;
		protected var allInstructionsEntities:Vector.<CompoundTextEntity> = new Vector.<CompoundTextEntity>();
		
		protected var slashKeysDown:Boolean;
		
		public function Player(playState:PlayState, x:Number, y:Number, keyLeft:int, keyRight:int, keyUp:int, keyDown:int, mouseActionMove:int, mouseActionTurn:int, moveMinDistance:Number, turnMinDistance:Number,
							   turnMaxAngle:Number, turnMaxAngleMaxDistance:Number, keyActionSlash:int, keyActionShoot:int, keyActionPickup:int, mouseActionSlash:int, mouseActionShoot:int, mouseActionPickup:int) {
			super(playState, x, y);

			this.keyLeft = keyLeft;
			this.keyRight = keyRight;
			this.keyUp = keyUp;
			this.keyDown = keyDown;
			//this.mouseActionMove = mouseActionMove;
			//this.mouseActionTurn = mouseActionTurn;
			//this.turnMaxAngle = turnMaxAngle;
			
			//this.moveMinDistance = moveMinDistance;
			//this.turnMinDistance = turnMinDistance;
			//this.turnMaxAngleMaxDistance = turnMaxAngleMaxDistance;
			
			this.keyActionSlash = keyActionSlash;
			this.keyActionShoot = keyActionShoot;
			this.keyActionPickup = keyActionPickup;
			//this.mouseActionSlash = mouseActionSlash;
			//this.mouseActionShoot = mouseActionShoot;
			//this.mouseActionPickup = mouseActionPickup;
			
			/*
			spritemap = new Spritemap(Assets.GUARDIAN, Assets.GUARDIAN_FRAMESIZE.x, Assets.GUARDIAN_FRAMESIZE.y);
			spritemap.originX = Assets.GUARDIAN_ORIGIN.x;
			spritemap.originY = Assets.GUARDIAN_ORIGIN.y;
			addGraphic(spritemap);
			
			spritemap.add(Assets.GUARDIAN_ANIMATION_SLASH_NAME, Assets.GUARDIAN_ANIMATION_SLASH_FRAMES, Assets.GUARDIAN_ANIMATION_SLASH_SPEED, false);
			spritemap.add(Assets.GUARDIAN_ANIMATION_WALK_NAME, Assets.GUARDIAN_ANIMATION_WALK_FRAMES, Assets.GUARDIAN_ANIMATION_WALK_SPEED);
			spritemap.play(Assets.GUARDIAN_ANIMATION_WALK_NAME);
			
			setHitbox(Assets.GUARDIAN_SIZE, Assets.GUARDIAN_SIZE, Assets.GUARDIAN_SIZE/2, Assets.GUARDIAN_SIZE/2);
			*/
			
			nextInstructionTextStartY = INSTRUCTION_TEXT_START_Y;
			
			var whiteColor:uint = 0xFFFFFF;
			
			instructionsMovement = addInstructionText(
				[["Slash a ", whiteColor],
					["crystal ", whiteColor],
					["to get ", whiteColor],
					["a little bit of instant health.", whiteColor]]);
			
			instructionsTextFadingOut = false;

			slashKeysDown = false;
		}
		
		protected function addInstructionText(content:Object):CompoundTextEntity
		{
			var entity:CompoundTextEntity = new CompoundTextEntity(0, 0, Constants.LAYER_GUI);
			entity.distance = -4;

			if (content is String)
			{
				content = [[content, 0xFFFFFF]];
			}
			
			for each (var contentParts:Array in content)
			{
				var text:Text = new Text(contentParts[0]);
				text.color = contentParts[1];
				text.align = "center";
				text.outlineColor = 0;
				text.outlineStrength = 4;
				
				entity.addTextAfter(text);
			}
			
			entity.x = FP.width / 2 - entity.width / 2;
			entity.y = nextInstructionTextStartY;
			
			playState.add(entity);
			allInstructionsEntities.push(entity);
			
			nextInstructionTextStartY += text.height + INSTRUCTION_TEXT_DISTANCE_Y;
			
			return entity;
		}
		
		protected override function updateIfRunning():void
		{
			var elapsedTime:Number = FP.elapsed;
			
			var direction:Point = new Point();

			var pressedCount:int = 0;
				
			if (Input.check(keyUp)) {
				direction.y--;
				pressedCount++;
			}
			if (Input.check(keyDown)) {
				direction.y++;
				pressedCount++;
			}
			if (Input.check(keyLeft)) {
				direction.x--;
				pressedCount++;
			}
			if (Input.check(keyRight)) {
				direction.x++;
				pressedCount++;
			}
			
			if (pressedCount == 2)
			{
				direction.normalize(1);
			}
		}
		
		protected function showText(string:String, x:Number, y:Number, color:uint = 0xFFFFFF):void
		{
			playState.add(new VanishingText(string, playState, x, y, Constants.LAYER_GUI, color));
		}
		
		public function hideInstructionsText(entity:CompoundTextEntity):void
		{
			if (entity == null)
				return;
			
			var fadeOutTween:VarTween = new VarTween(
				function():void
				{
					playState.remove(entity);
				},
				Tween.ONESHOT);
			
			fadeOutTween.tween(entity, "alpha", 0, 0.5);
			entity.active = true;
			entity.addTween(fadeOutTween);
			
			var entityIndex:int = allInstructionsEntities.indexOf(entity);
			if (entityIndex == -1)
				return;
			
			allInstructionsEntities.splice(entityIndex, 1);
			
			var textHeight:Number = entity.height;
			
			var textPosY:Number = INSTRUCTION_TEXT_START_Y + (textHeight + INSTRUCTION_TEXT_DISTANCE_Y) * entityIndex;
			
			for (var i:int = entityIndex; i < allInstructionsEntities.length; i++)
			{
				var movingEntity:CompoundTextEntity = allInstructionsEntities[i];
				movingEntity.clearTweens();
				
				var moveTween:VarTween = new VarTween(null, Tween.ONESHOT);
				moveTween.tween(movingEntity, "y", textPosY, 0.5, Ease.backIn);
				movingEntity.addTween(moveTween);
				movingEntity.active = true;
				
				textPosY += textHeight + INSTRUCTION_TEXT_DISTANCE_Y;
			}
		}
		
		override public function render():void
		{
			//super.render();
			Draw.circle(x, y, 10, 0xFF0000);
		}
	}
}
