package game.startState
{
	import flash.net.URLRequest;
	import flash.net.navigateToURL;
	
	import game.Assets;
	import game.Constants;
	import game.general.ClickableText;
	import game.playState.CompoundTextEntity;
	import game.playState.Overlay;
	import game.playState.PlayState;
	
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.Tween;
	import net.flashpunk.World;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.tweens.misc.VarTween;
	import net.flashpunk.utils.Ease;
	
	public class StartState extends World
	{
		protected var backgroundEntity:Entity;
		
		protected var creditsEntity:CompoundTextEntity;
		
		protected var controlScheme1:ClickableText;
		protected var controlScheme2:ClickableText;
		protected var overlay:Overlay;
		
		protected var compatibiltyModeText:ClickableText;
		protected var compatibiltyModeOverlay:Overlay;
		
		public function StartState()
		{
			super();
		
			var backgroundImage:Image = new Image(Assets.START_BACKGROUND);
			backgroundEntity = addGraphic(backgroundImage, Constants.LAYER_BACKGROUND);
			
			var creditsSpacer:Number = 4;
			
			var textLeft:Text = new Text("Art and Sound:\nProgramming:\nMusic:");
			textLeft.align = "right";
			textLeft.outlineColor = 0;
			textLeft.outlineStrength = 4;

			var textRight:Text = new Text("Michael Kessler\nTobias Wehrum\nKevin MacLeod");
			textRight.x = textLeft.width + creditsSpacer;
			textRight.align = "left";
			textRight.outlineColor = 0;
			textRight.outlineStrength = 4;

			creditsEntity = new CompoundTextEntity(0, 0, Constants.LAYER_TEXT_OVER_OVERLAY);
			creditsEntity.addGraphic(textRight);
			creditsEntity.addGraphic(textLeft);
			creditsEntity.width = textRight.x + textRight.width;
			creditsEntity.height = textLeft.height;
			creditsEntity.x = FP.width - creditsEntity.width;
			creditsEntity.y = FP.height - creditsEntity.height;
			creditsEntity.alpha = 0;
			add(creditsEntity);

			var controlSchemeY:Number = 442;
			var overlayHeight:Number = 0;
			
			if (Constants.COMPATIBILITY_MODE)
			{
				controlScheme1 = new ClickableText("Compatibilty Control Scheme 1:\nMove: WASD keys\nSlash: Left click\nShoot: Shift\nPick up/Drop: Space", onControlScheme3);
				controlScheme1.layer = Constants.LAYER_TEXT_OVER_OVERLAY;
				controlScheme1.x = FP.width / 4 - controlScheme1.width / 2;
				controlScheme1.y = controlSchemeY;
				add(controlScheme1);

				controlScheme2 = new ClickableText("Compatibilty Control Scheme 2:\nMove: Mouse pointer\nSlash: Left click or X\nShoot: C\nPick up/Drop: V", onControlScheme4);
				controlScheme2.layer = Constants.LAYER_TEXT_OVER_OVERLAY;
				controlScheme2.x = FP.width / 4 * 3 - controlScheme2.width / 2;
				controlScheme2.y = controlSchemeY;
				add(controlScheme2);
				
				compatibiltyModeText = new ClickableText("Your flash version is quite old!\nYou cannot use features like the right and middle click.\nClick here to visit the official Adobe update page.", onUpdateLink);
				compatibiltyModeText.y = FP.height - compatibiltyModeText.height;
				compatibiltyModeText.textField.color = 0xFF8080;
				add(compatibiltyModeText);
				
				overlayHeight = FP.height - controlSchemeY;
			}
			else
			{
				controlScheme1 = new ClickableText("Control Scheme 1:\nMove: WASD keys\nSlash: Left click\nShoot: Right click\nPick up/Drop: Middle Click or Space", onControlScheme1);
				controlScheme1.layer = Constants.LAYER_TEXT_OVER_OVERLAY;
				controlScheme1.x = FP.width / 4 - controlScheme1.width / 2;
				controlScheme1.y = controlSchemeY;
				add(controlScheme1);

				controlScheme2 = new ClickableText("Control Scheme 2:\nMove: Mouse pointer\nSlash: Left click or X\nShoot: Right click or C\nPick up/Drop: Middle click or V", onControlScheme2);
				controlScheme2.layer = Constants.LAYER_TEXT_OVER_OVERLAY;
				controlScheme2.x = FP.width / 4 * 3 - controlScheme2.width / 2;
				controlScheme2.y = controlSchemeY;
				add(controlScheme2);
				
				overlayHeight = controlScheme1.height;
			}
			
			var border:Number = 10;
			overlay = new Overlay(0);
			overlay.y = controlScheme1.y - border;
			overlay.color = 0xFFFFFF;
			overlay.height = overlayHeight + border * 2;
			add(overlay);

			var fadeTime:Number = 1;
			
			var overlayTween:VarTween = new VarTween(null, Tween.ONESHOT);
			overlayTween.tween(overlay, "alpha", 0.75, fadeTime);
			addTween(overlayTween);
			
			var moveDistance:Number = Math.max(controlScheme1.x + controlScheme1.width, (controlScheme2 != null) ? (FP.width - controlScheme2.x) : 0);
			
			var moveTime:Number = 1;
			var moveEase:Function = Ease.backOut;
			
			controlScheme1.x -= moveDistance;
			var controlScheme1Tween:VarTween = new VarTween(null, Tween.ONESHOT);
			controlScheme1Tween.tween(controlScheme1, "x", controlScheme1.x + moveDistance, moveTime, moveEase);
			addTween(controlScheme1Tween);

			if (controlScheme2 != null)
			{
				controlScheme2.x += moveDistance;
				
				var controlScheme2Tween:VarTween = new VarTween(null, Tween.ONESHOT);
				controlScheme2Tween.tween(controlScheme2, "x", controlScheme2.x - moveDistance, moveTime, moveEase);
				addTween(controlScheme2Tween);
			}
			
			if (compatibiltyModeText != null)
			{
				var compatibiltyModeTextMoveDistance:Number = compatibiltyModeText.x + compatibiltyModeText.width;
				
				compatibiltyModeText.x -= compatibiltyModeTextMoveDistance;
				var compatibiltyModeTextTween:VarTween = new VarTween(null, Tween.ONESHOT);
				compatibiltyModeTextTween.tween(compatibiltyModeText, "x", compatibiltyModeText.x + compatibiltyModeTextMoveDistance, moveTime, moveEase);
				addTween(compatibiltyModeTextTween);
			}
			
			var creditsTween:VarTween = new VarTween(null, Tween.ONESHOT);
			creditsTween.tween(creditsEntity, "alpha", 1, fadeTime);
			addTween(creditsTween);

			//add(new Cursor());
		}
		
		public function fadeOut(f:Function):void
		{
			var fadeTime:Number = 1;
			
			var overlayTween:VarTween = new VarTween(f, Tween.ONESHOT);
			overlayTween.tween(overlay, "alpha", 0, fadeTime);
			addTween(overlayTween);
			
		
			var moveDistance:Number = Math.max(controlScheme1.x + controlScheme1.width, (controlScheme2 != null) ? (FP.width - controlScheme2.x) : 0);
			var moveTime:Number = 1;
			var moveEase:Function = Ease.backIn;
			
			var controlScheme1Tween:VarTween = new VarTween(null, Tween.ONESHOT);
			controlScheme1Tween.tween(controlScheme1, "x", controlScheme1.x - moveDistance, moveTime, moveEase);
			addTween(controlScheme1Tween);
			
			if (controlScheme2 != null)
			{
				var controlScheme2Tween:VarTween = new VarTween(null, Tween.ONESHOT);
				controlScheme2Tween.tween(controlScheme2, "x", controlScheme2.x + moveDistance, moveTime, moveEase);
				addTween(controlScheme2Tween);
			}

			if (compatibiltyModeText != null)
			{
				var compatibiltyModeTextMoveDistance:Number = compatibiltyModeText.x + compatibiltyModeText.width;
				
				var compatibiltyModeTextTween:VarTween = new VarTween(null, Tween.ONESHOT);
				compatibiltyModeTextTween.tween(compatibiltyModeText, "x", compatibiltyModeText.x - compatibiltyModeTextMoveDistance, moveTime, moveEase);
				addTween(compatibiltyModeTextTween);
			}
			
			var creditsTween:VarTween = new VarTween(null, Tween.ONESHOT);
			creditsTween.tween(creditsEntity, "alpha", 0, fadeTime);
			addTween(creditsTween);
		}
		
		public function onControlScheme1(target:ClickableText):void
		{
			fadeOut(function():void { startGame(1); });
		}
		
		public function onControlScheme2(target:ClickableText):void
		{
			fadeOut(function():void { startGame(2); });
		}
		
		public function onControlScheme3(target:ClickableText):void
		{
			fadeOut(function():void { startGame(3); });
		}
		
		public function onControlScheme4(target:ClickableText):void
		{
			fadeOut(function():void { startGame(4); });
		}
		
		public function onUpdateLink(target:ClickableText):void
		{
			navigateToURL(new URLRequest("http://get.adobe.com/flashplayer"), "_blank");
		}

		public function startGame(controlScheme:uint):void
		{
			FP.world = new PlayState(true, controlScheme);
		}
	}
}