package helper
{
	import flash.display.DisplayObject;
	import flash.geom.Point;
	import flash.geom.Vector3D;
	
	import net.flashpunk.Entity;

	/**
	 * A static class containing mathematical and geometrical helper functions
	 * @author Tobias
	 */	
	public class MathHelper {
		/**
		 * Multiplicate with this value to convert from rad to grad 
		 */		
		public static const RAD_TO_GRAD:Number = 180 / Math.PI;

		/**
		 * Multiplicate with this value to convert from grad to rad 
		 */		
		public static const GRAD_TO_RAD:Number = Math.PI / 180;
		
		/**
		 * Calculates the distance between a and b
		 * @param a a display object
		 * @param b a point
		 * @return the distance
		 */		
		public static function getDistanceDisplayObjectPoint(a:DisplayObject, b:Point):Number {
			var dx:Number = a.x - b.x;
			var dy:Number = a.y - b.y;
			return Math.sqrt(dx*dx + dy*dy);
		}

		public static function getDistanceDisplayObjectDisplayObject(a:DisplayObject, b:DisplayObject):Number {
			var dx:Number = a.x - b.x;
			var dy:Number = a.y - b.y;
			return Math.sqrt(dx*dx + dy*dy);
		}
		
		public static function getDistanceEntityEntity(a:Entity, b:Entity):Number {
			var dx:Number = a.x - b.x;
			var dy:Number = a.y - b.y;
			return Math.sqrt(dx*dx + dy*dy);
		}

		public static function getDistanceEntityEntitySqrt(a:Entity, b:Entity):Number {
			var dx:Number = a.x - b.x;
			var dy:Number = a.y - b.y;
			return dx*dx + dy*dy;
		}

		/**
		 * Calculates the distance^2 between a and b. This method is faster because it doesn't have to call Math.sqrt.
		 * @param a a display object
		 * @param b a point
		 * @return the distance^2
		 */		
		public static function getDistanceDisplayObjectPointSqrt(a:DisplayObject, b:Point):Number {
			var dx:Number = a.x - b.x;
			var dy:Number = a.y - b.y;
			return dx*dx + dy*dy;
		}

		/**
		 * Calculates the distance between a and b
		 * @param a a point
		 * @param b a point
		 * @return the distance
		 */		
		public static function getDistancePointPoint(a:Point, b:Point):Number {
			var dx:Number = a.x - b.x;
			var dy:Number = a.y - b.y;
			return Math.sqrt(dx*dx + dy*dy);
		}
		
		/**
		 * Calculates the distance^2 between a and b. This method is faster because it doesn't have to call Math.sqrt.
		 * @param a a point
		 * @param b a point
		 * @return the distance^2
		 */		
		public static function getDistancePointPointSqrt(a:Point, b:Point):Number {
			var dx:Number = a.x - b.x;
			var dy:Number = a.y - b.y;
			return dx*dx + dy*dy;
		}

		/**
		 * Calculates the distance between a and b
		 * @param a a point
		 * @param b a point
		 * @return the distance
		 */		
		public static function getDistance(ax:Number, ay:Number, bx:Number, by:Number):Number {
			var dx:Number = ax - bx;
			var dy:Number = ay - by;
			return Math.sqrt(dx*dx + dy*dy);
		}
		
		/**
		 * Calculates the distance^2 between a and b. This method is faster because it doesn't have to call Math.sqrt.
		 * @param a a point
		 * @param b a point
		 * @return the distance^2
		 */		
		public static function getDistanceSqrt(ax:Number, ay:Number, bx:Number, by:Number):Number {
			var dx:Number = ax - bx;
			var dy:Number = ay - by;
			return dx*dx + dy*dy;
		}

		public static function getRandomIntInRange(minInclusive:int, maxExclusive:int):int {
			return minInclusive + Math.floor(Math.random() * (maxExclusive - minInclusive));
		}
		
		public static function getRandomInRange(minInclusive:Number, maxExclusive:Number):Number {
			return minInclusive + Math.random() * (maxExclusive - minInclusive);
		}
		
		public static function clamp(value:Number, min:Number, max:Number):Number {
			return Math.min(Math.max(value, min), max);
		}
		
		public static function sign(value:Number):int {
			if (value < 0) {
				return -1;
			} else if (value > 0) {
				return 1;
			}
			return 0;
		}
		
		// http://actionsnippet.com/?p=1494
		/**
		 Original function by Pieter Iserbyt:
		 http://local.wasp.uwa.edu.au/~pbourke/geometry/pointline/DistancePoint.java
		 from Paul Bourke's website:
		 http://local.wasp.uwa.edu.au/~pbourke/geometry/pointline/
		 */
		public static function getDistancePointLineSegment(p:Point, lineP1:Point, lineP2:Point):Number {
			var xDelta:Number = lineP2.x - lineP1.x;
			var yDelta:Number = lineP2.y - lineP1.y;
			if ((xDelta == 0) && (yDelta == 0)) {
				// p1 and p2 cannot be the same point
				lineP2.x += 1;
				lineP2.y += 1;
				xDelta = 1;
				yDelta = 1;
			}
			var u:Number = ((p.x - lineP1.x) * xDelta + (p.y - lineP1.y) * yDelta) / (xDelta * xDelta + yDelta * yDelta);
			var closestPoint:Point;
			if (u <0) {
				closestPoint = new Point(lineP1.x, lineP1.y);
			} else if (u> 1) {
				closestPoint = new Point(lineP2.x, lineP2.y);
			} else {
				closestPoint = new Point(lineP1.x + u * xDelta, lineP1.y + u * yDelta);
			}
			return Point.distance(closestPoint, p);
		}
		
		/**
		 * Find the intersection of this circle with the line defined by point p1 & p2
		 */
		// See: http://www.experts-exchange.com/Programming/Languages/Java/Q_21169189.html
		// Furthermore see: http://mathworld.wolfram.com/Circle-LineIntersection.html
		public static function circleIntersections(circleCenter:Point, radius:Number, lineP1:Point, lineP2:Point, lineSeg:Boolean):Vector.<Point> {
			var p1:Point = new Point(lineP1.x - circleCenter.x, lineP1.y - circleCenter.y);
			var p2:Point = new Point(lineP2.x - circleCenter.x, lineP2.y - circleCenter.y);
			var dx:Number = p2.x - p1.x;
			var dy:Number = p2.y - p1.y;
			var dr:Number = Math.sqrt(dx * dx + dy * dy);
			var D:Number = p1.x * p2.y - p2.x * p1.y;
			var discriminant:Number = radius * radius * dr * dr - D * D;
			
			if (discriminant < 0)
				return new Vector.<Point>(); // No intersection;
			
			/*
			* (discriminant == 0) // Tangant line there is only ONE intersection
			* (discriminant > 0) // The line intersects at TWO points
			*/
			
			// Compute intersection for ONE point (to compute intersection at OTHER
			// point change + to a -)
			
			var lineSegDistTolerance:Number = 0.001;
			
			var x:Number;
			var y:Number;
			var p:Point;
			
			var list:Vector.<Point> = new Vector.<Point>();
			if (discriminant >= 0) {
				// at least one intersection (discriminant == 0 would be exactly one)
				x = (D * dy + sgn(dy) * dx * Math.sqrt(discriminant)) / (dr * dr);
				y = (-D * dx + Math.abs(dy) * Math.sqrt(discriminant)) / (dr * dr);
				x += circleCenter.x;
				y += circleCenter.y;
				
				p = new Point(x, y);
				if ((!lineSeg) || (getDistancePointLineSegment(p, lineP1, lineP2) <= lineSegDistTolerance)) {
					list.push(p);
				}
			}
			
			if (discriminant > 0) {
				// ...and > 0 means TWO!
				x = (D * dy - sgn(dy) * dx * Math.sqrt(discriminant)) / (dr * dr);
				y = (-D * dx - Math.abs(dy) * Math.sqrt(discriminant)) / (dr * dr);
				x += circleCenter.x;
				y += circleCenter.y;
				
				p = new Point(x, y);
				if ((!lineSeg) || (getDistancePointLineSegment(p, lineP1, lineP2) <= lineSegDistTolerance)) {
					list.push(p);
				}
			}
			return list;
		}

		public static function sgn(number:Number):Number {
			return ((number >= 0) ? 1 : -1);
		}
		
		public static function randomNumber(minInclusive:Number, maxExclusive:Number):Number {
			return minInclusive + Math.random() * (maxExclusive-minInclusive);
		}
		
		public static function randomInt(minInclusive:int, maxExclusive:int):int {
			return Math.floor(minInclusive + Math.random() * (maxExclusive-minInclusive));
		}

		public static function directionToFPAngle(direction:Point):Number
		{
			return Math.atan2(direction.y, direction.x) * MathHelper.RAD_TO_GRAD * -1 + 90;
		}
		
		public static function fpAngleToDirection(angle:Number, outDirection:Point):Point
		{
			if (outDirection == null)
				outDirection = new Point();
			
			var revertedRadAngle:Number = (angle - 90) * -MathHelper.GRAD_TO_RAD;
			
			outDirection.x = Math.cos(revertedRadAngle);
			outDirection.y = Math.sin(revertedRadAngle);
			
			return outDirection;
		}
		
		public static function normalizeAngleGrad(angle:Number):Number
		{
			while (angle < 0)
				angle += 360;
			
			if (angle > 360)
				angle %= 360;
			
			return angle;
		}
		
		public static function getAngleDifference(angle1:Number, angle2:Number):Number
		{
			var difference:Number = normalizeAngleGrad(angle2 - angle1);
			
			if (difference > 180)
				difference = -(360 - difference);
			
			return difference;
		}
	}
}